#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from onf_website.workflow import VISIBLE_STATES
from pyams_utils.url import absolute_url

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from hypatia.interfaces import ICatalog
from onf_website.component.links.interfaces import IBaseLink, IInternalLink, IExternalLink
from onf_website.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IFormContextPermissionChecker
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent

# import packages
from hypatia.catalog import CatalogQuery
from hypatia.query import Eq, Any
from persistent import Persistent
from pyams_catalog.query import CatalogResultSet
from pyams_sequence.utility import get_last_version
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.lifecycleevent import ObjectModifiedEvent
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


@implementer(IBaseLink)
class BaseLink(Persistent, Contained):
    """Base link persistent class"""

    title = FieldProperty(IBaseLink['title'])
    description = FieldProperty(IBaseLink['description'])


@adapter_config(context=IBaseLink, provides=IFormContextPermissionChecker)
class BaseLinkPermissionChecker(ContextAdapter):
    """Base link permission checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IWfSharedContent)
        return IFormContextPermissionChecker(content).edit_permission


@subscriber(IObjectAddedEvent, context_selector=IBaseLink)
def handle_added_link(event):
    """Handle added link"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IBaseLink)
def handle_modified_link(event):
    """Handle modified link"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IBaseLink)
def handle_removed_link(event):
    """Handle removed link"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@implementer(IInternalLink)
class InternalLink(BaseLink):
    """Internal link persistent class"""

    reference = FieldProperty(IInternalLink['reference'])

    def get_target(self, state=None):
        catalog = get_utility(ICatalog)
        params = Eq(catalog['oid'], self.reference)
        if state:
            if not isinstance(state, (list, tuple)):
                state = (state, )
            params &= Any(catalog['workflow_state'], state)
            results = list(CatalogResultSet(CatalogQuery(catalog).query(params)))
            if results:
                return results[0]
        else:
            results = list(map(get_last_version, CatalogResultSet(CatalogQuery(catalog).query(params))))
            if results:
                return results[0]

    def get_editor_url(self):
        return 'oid://{0}'.format(self.reference)

    def get_url(self, request, view_name=None):
        target = self.get_target(state=VISIBLE_STATES)
        if target is not None:
            return absolute_url(target, request, view_name)
        else:
            return ''


@implementer(IExternalLink)
class ExternalLink(BaseLink):
    """external link persistent class"""

    url = FieldProperty(IExternalLink['url'])
    language = FieldProperty(IExternalLink['language'])

    def get_editor_url(self):
        return self.url

    def get_url(self, request, view_name=None):
        return self.url
