#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from hypatia.interfaces import ICatalog
from onf_website.profile.interfaces import IAdminProfile
from onf_website.root.interfaces import ISiteRoot
from onf_website.shared.common.interfaces import ISharedTool
from onf_website.shared.common.interfaces.zmi import ISharedToolDashboardTable, ISiteRootDashboardTable
from onf_website.zmi.interfaces import IDashboardMenu
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.interfaces.configuration import IConfiguration
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowVersions, IWorkflowState
from pyams_zmi.interfaces.menu import IContentManagementMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IValues, IColumn
from zope.dublincore.interfaces import IZopeDublinCore

# import packages
from hypatia.catalog import CatalogQuery
from hypatia.query import Any
from onf_website.shared.common import CONTENT_TYPES
from onf_website.workflow import WAITING_STATES
from pyams_catalog.query import CatalogResultSet
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, I18nColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.list import unique
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.view import AdminView
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from onf_website import _


@viewlet_config(name='dashboard.menu', context=ISiteRoot, layer=IAdminLayer,
                manager=IContentManagementMenu, permission='system.view', weight=1)
@viewletmanager_config(name='dashboard.menu', layer=IAdminLayer, provides=IDashboardMenu)
@implementer(IDashboardMenu)
class SiteRootDashboardMenu(MenuItem):
    """Site root dashboard menu"""

    label = _("Dashboard")
    icon_class = 'fa-line-chart'
    url = '#dashboard.html'


@pagelet_config(name='dashboard.html', context=ISiteRoot, layer=IPyAMSLayer, permission='system.view')
@template_config(template='templates/dashboard.pt', layer=IAdminLayer)
@implementer(IInnerPage)
class SiteRootDashboardView(AdminView):
    """Site root dashboard view"""

    @property
    def title(self):
        return II18n(self.context).query_attribute('title', request=self.request)

    def __init__(self, context, request):
        super(SiteRootDashboardView, self).__init__(context, request)
        self.waiting_table = SiteRootDashboardWaitingTable(self.context, self.request)
        self.modified_table = SiteRootDashboardModifiedTable(self.context, self.request)

    def update(self):
        super(SiteRootDashboardView, self).update()
        self.waiting_table.update()
        self.modified_table.update()


@adapter_config(context=(ISiteRoot, IPyAMSLayer, SiteRootDashboardView), provides=IPageHeader)
class SiteRootDashboardHeaderAdapter(DefaultPageHeaderAdapter):
    """Site root properties header adapter"""

    icon_class = 'fa fa-fw fa-line-chart'

    @property
    def title(self):
        config = IConfiguration(self.context)
        return II18n(config).query_attribute('title', request=self.request)

    subtitle = _("Contents dashboard")


@implementer(ISiteRootDashboardTable)
class SiteRootDashboardWaitingTable(BaseTable):
    """Site root dashboard waiting contents table"""

    title = _("Contents waiting for action")
    hide_toolbar = True
    sortOn = None

    @property
    def data_attributes(self):
        attributes = super(SiteRootDashboardWaitingTable, self).data_attributes
        attributes['table'] = {'data-ams-datatable-sorting': '[]',
                               'data-ams-datatable-display-length':
                                   IAdminProfile(self.request.principal).table_page_length}
        return attributes


@adapter_config(context=(ISiteRoot, IPyAMSLayer, SiteRootDashboardWaitingTable), provides=IValues)
class SiteRootDashboardWaitingValues(ContextRequestViewAdapter):
    """Site root dashboard waiting contents values adapter"""

    @property
    def values(self):
        catalog = query_utility(ICatalog)
        params = Any(catalog['content_type'], CONTENT_TYPES.keys()) & \
                 Any(catalog['workflow_state'], WAITING_STATES)
        return unique(map(lambda x: sorted(IWorkflowVersions(x).get_versions(IWorkflowState(x).state),
                                           key=lambda y: IZopeDublinCore(y).modified, reverse=True)[0],
                          CatalogResultSet(CatalogQuery(catalog).query(params,
                                                                       sort_index='modified_date'))))


@implementer(ISiteRootDashboardTable)
class SiteRootDashboardModifiedTable(BaseTable):
    """Site root dashboard modified contents table"""

    title = _("Last modified contents (limited to 30)")
    hide_toolbar = True
    sortOn = None

    @property
    def data_attributes(self):
        attributes = super(SiteRootDashboardModifiedTable, self).data_attributes
        attributes['table'] = {'data-ams-datatable-sorting': '[]',
                               'data-ams-datatable-display-length':
                                   IAdminProfile(self.request.principal).table_page_length}
        return attributes


@adapter_config(context=(ISiteRoot, IPyAMSLayer, SiteRootDashboardModifiedTable), provides=IValues)
class SiteRootDashboardModifiedValues(ContextRequestViewAdapter):
    """Site root dashboard modified contents values adapter"""

    @property
    def values(self):
        catalog = query_utility(ICatalog)
        params = Any(catalog['content_type'], CONTENT_TYPES.keys())
        return unique(map(lambda x: IWorkflowVersions(x).get_last_versions()[0],
                          CatalogResultSet(CatalogQuery(catalog).query(params,
                                                                       limit=30,
                                                                       sort_index='modified_date',
                                                                       reverse=True))))


@adapter_config(name='tool', context=(Interface, IPyAMSLayer, ISiteRootDashboardTable), provides=IColumn)
class SiteRootDashboardToolColumn(I18nColumn, GetAttrColumn):
    """Shared tool dashboard sequence ID column"""

    _header = _("Tool")
    weight = 1

    def getValue(self, obj):
        tool = get_parent(obj, ISharedTool)
        return II18n(tool).query_attribute('short_name', request=self.request)
