#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.base.interfaces import IBaseContentInfo
from onf_website.shared.common.interfaces import IWfSharedContent, IWfSharedContentRoles, ISharedContent
from pyams_i18n.interfaces import II18nManager
from pyams_security.interfaces import IDefaultProtectionPolicy
from pyams_sequence.interfaces import ISequentialIdTarget, ISequentialIdInfo
from pyams_workflow.interfaces import IWorkflowPublicationSupport, IWorkflow, IWorkflowManagedContent
from zope.dublincore.interfaces import IZopeDublinCore

# import packages
from persistent import Persistent
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObject
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from zope.container.contained import Contained
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty


#
# Content types management
#

CONTENT_TYPES = {}

def register_content_type(content):
    """Register a new content type"""
    CONTENT_TYPES[content.content_type] = content


@implementer(IDefaultProtectionPolicy, IWfSharedContent, IWfSharedContentRoles, IWorkflowPublicationSupport,
             II18nManager)
class WfSharedContent(ProtectedObject, Persistent, Contained):
    """Shared data content class"""

    __roles__ = ('onf.Reader', )
    roles_interface = IWfSharedContentRoles

    readers = RolePrincipalsFieldProperty(IWfSharedContentRoles['readers'])

    content_type = None

    title = FieldProperty(IWfSharedContent['title'])
    short_name = FieldProperty(IWfSharedContent['short_name'])
    owner = FieldProperty(IWfSharedContent['owner'])
    description = FieldProperty(IWfSharedContent['description'])

    languages = FieldProperty(II18nManager['languages'])


@adapter_config(context=IWfSharedContent, provides=ISequentialIdInfo)
def WfSharedContentSequenceAdapter(context):
    """Shared content sequence adapter"""
    parent = get_parent(context, ISharedContent)
    return ISequentialIdInfo(parent)


@adapter_config(context=IWfSharedContent, provides=IBaseContentInfo)
class WfSharedContentInfoAdapter(ContextAdapter):
    """Shared content base info adapter"""

    @property
    def created_date(self):
        return IZopeDublinCore(self.context).created

    @property
    def modified_date(self):
        return IZopeDublinCore(self.context).modified


@adapter_config(context=IWfSharedContent, provides=IWorkflow)
def WfSharedContentWorkflowAdapter(context):
    """Shared content workflow adapter"""
    content = get_parent(context, IWorkflowManagedContent)
    return query_utility(IWorkflow, name=content.workflow_name)


@implementer(ISharedContent, ISequentialIdTarget)
class SharedContent(Persistent, Contained):
    """Workflow managed shared data"""

    workflow_name = 'ONF website workflow'
    view_permission = 'view'

    sequence_name = ''  # use default sequence generator
    sequence_prefix = ''


@adapter_config(context=ISharedContent, provides=IBaseContentInfo)
class SharedContentInfoAdapter(ContextAdapter):
    """Shared content base info adapter"""

    @property
    def created_date(self):
        return IZopeDublinCore(self.context).created

    @property
    def modified_date(self):
        return IZopeDublinCore(self.context).modified


@adapter_config(context=ISharedContent, provides=IWorkflow)
def SharedContentWorkflowAdapter(context):
    """Shared content workflow adapter"""
    return query_utility(IWorkflow, name=context.workflow_name)
