#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from pyams_utils.timezone import tztime

__docformat__ = 'restructuredtext'


# import standard library
from datetime import datetime
from uuid import uuid4

# import interfaces
from onf_website.shared.common.interfaces import IWfSharedContent, ISharedContent, ISharedTool
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_i18n.interfaces import II18n
from pyams_sequence.interfaces import ISequentialIntIds, ISequentialIdInfo
from pyams_skin.interfaces import IContentTitle, IContentHelp
from pyams_skin.interfaces.container import ITable, ITableElementEditor
from pyams_skin.interfaces.viewlet import IContextActions
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowVersions, IWorkflowInfo, IWorkflowState, IWorkflowCommentInfo, IWorkflow
from pyams_zmi.layer import IAdminLayer
from zope.dublincore.interfaces import IZopeDublinCore

# import packages
from onf_website.workflow import READONLY_STATES, PROTECTED_STATES, DRAFT
from pyams_form.schema import CloseButton
from pyams_form.form import AJAXAddForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.help import ContentHelp
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import DefaultElementEditorAdapter
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter, ContextAdapter
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_workflow.versions import WorkflowHistoryItem
from pyams_zmi.form import AdminDialogAddForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.copy import copy
from zope.interface import Interface
from zope.lifecycleevent import ObjectCreatedEvent

from onf_website import _


class SharedContentAddForm(AdminDialogAddForm):
    """Shared content add form"""

    @property
    def title(self):
        return II18n(self.context).query_attribute('title', request=self.request)

    fields = field.Fields(IWfSharedContent).select('title', 'description')

    ajax_handler = 'add-shared-content.json'
    edit_permission = 'onf.CreateContent'

    content_class = None
    wf_content_class = None

    def updateWidgets(self, prefix=None):
        super(SharedContentAddForm, self).updateWidgets(prefix)
        self.widgets['description'].label_css_class = 'textarea'

    def create(self, data):
        return self.wf_content_class()

    def update_content(self, content, data):
        changes = super(SharedContentAddForm, self).update_content(content, data)
        content.owner = self.request.principal.id
        content.short_name = content.title.copy()
        return changes

    def add(self, wf_content):
        content = self.content_class()
        self.request.registry.notify(ObjectCreatedEvent(content))
        uuid = self.__uuid = str(uuid4())
        self.context[uuid] = content
        IWorkflowVersions(content).add_version(wf_content, None)
        IWorkflowInfo(wf_content).fire_transition('init')

    def nextURL(self):
        return absolute_url(self.context, self.request, '{0}/++versions++/1/@@admin.html'.format(self.__uuid))


class SharedContentAJAXAddForm(AJAXAddForm):
    """Shared event add form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'redirect',
                'location': self.nextURL()}


#
# Edit adapters and views
#

@adapter_config(context=IWfSharedContent, provides=IFormContextPermissionChecker)
class WfSharedContentPermissionChecker(ContextAdapter):
    """Shared content form permission checker"""

    @property
    def edit_permission(self):
        state = IWorkflowState(self.context).state
        if state in READONLY_STATES:
            return 'system.forbidden'
        elif state in PROTECTED_STATES:
            return 'onf.ManageSiteRoot'
        else:
            return 'onf.ManageContent'


@adapter_config(context=(IWfSharedContent, IPyAMSLayer, Interface), provides=IContentTitle)
class WfSharedContentTitleAdapter(ContextRequestViewAdapter):
    """Shared content title adapter"""

    @property
    def title(self):
        return II18n(self.context).query_attribute('title', request=self.request)


class WfSharedContentHeaderAdapter(DefaultPageHeaderAdapter):
    """Shared content header adapter"""

    @property
    def back_url(self):
        shared_tool = get_parent(self.context, ISharedTool)
        return absolute_url(shared_tool, self.request, 'admin.html#dashboard.html')

    back_target = None
    icon_class = 'fa fa-fw fa-edit'


@adapter_config(context=(IWfSharedContent, IPyAMSLayer, ITable), provides=ITableElementEditor)
class WfSharedContentElementEditor(DefaultElementEditorAdapter):
    """Shared content element editor"""

    view_name = 'admin.html'
    modal_target = False


#
# Duplication menus and views
#

@viewlet_config(name='duplication.menu', context=IWfSharedContent, layer=IPyAMSLayer,
                manager=IContextActions, permission='onf.CreateContent', weight=1)
class WfSharedContentDuplicateMenu(ToolbarMenuItem):
    """Shared content duplication menu item"""

    label = _("Duplicate content...")
    label_css_class = 'fa fa-fw fa-files-o'

    url = 'duplicate.html'
    modal_target = True


class ISharedContentDuplicateButtons(Interface):
    """Shared content duplication form buttons"""

    close = CloseButton(name='close', title=_("Close"))
    duplicate = button.Button(name='duplicate', title=_("Duplicate content"))


@pagelet_config(name='duplicate.html', context=IWfSharedContent, layer=IPyAMSLayer, permission='onf.CreateContent')
class WfSharedContentDuplicateForm(AdminDialogAddForm):
    """Shared content duplicate form"""

    legend = _("Duplicate content")
    fields = field.Fields(IWorkflowCommentInfo)
    buttons = button.Buttons(ISharedContentDuplicateButtons)

    ajax_handler = 'duplicate.json'
    edit_permission = 'onf.CreateContent'

    def updateWidgets(self, prefix=None):
        super(WfSharedContentDuplicateForm, self).updateWidgets(prefix)
        self.widgets['comment'].label_css_class = 'textarea'

    def updateActions(self):
        super(WfSharedContentDuplicateForm, self).updateActions()
        if 'duplicate' in self.actions:
            self.actions['duplicate'].addClass('btn-primary')

    def createAndAdd(self, data):
        registry = self.request.registry
        # initialize new content
        content = get_parent(self.context, ISharedContent)
        new_content = content.__class__()
        registry.notify(ObjectCreatedEvent(new_content))
        container = get_parent(content, ISharedTool)
        container[str(uuid4())] = new_content
        # initialize new version
        new_version = copy(self.context)
        registry.notify(ObjectCreatedEvent(new_version))
        new_version.owner = self.request.principal.id
        # store new version
        translate = self.request.localizer.translate
        workflow = get_utility(IWorkflow, name=new_content.workflow_name)
        sequence = get_utility(ISequentialIntIds, name=new_content.sequence_name)
        IWorkflowVersions(new_content).add_version(new_version, DRAFT)
        history = WorkflowHistoryItem(date=datetime.utcnow(),
                                      source_state=translate(workflow.states.getTerm(IWorkflowState(
                                          self.context).state).title),
                                      transition=translate(_("Duplicate content ({oid})")).format(
                                          oid=sequence.get_short_oid(ISequentialIdInfo(content).oid,
                                                                     content.sequence_prefix)),
                                      target_state=translate(workflow.states.getTerm(DRAFT).title),
                                      principal=self.request.principal.id,
                                      comment=data.get('comment'))
        state = IWorkflowState(new_version)
        state.history.clear()
        state.history.append(history)
        return new_version


@view_config(name='duplicate.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.CreateContent', renderer='json', xhr=True)
class WfSharedContentDuplicationAJAXForm(AJAXAddForm, WfSharedContentDuplicateForm):
    """Shared content duplicate form, JSON renderer"""

    def get_ajax_output(self, changes):
        return {'status': 'redirect',
                'location': absolute_url(changes, self.request, 'admin.html')}


@adapter_config(context=(IWfSharedContent, IAdminLayer, WfSharedContentDuplicateForm), provides=IContentHelp)
class WfSharedContentDuplicationHelpAdapter(ContentHelp):
    """Shared content duplication form help adapter"""

    header = _("WARNING")
    status = 'danger'
    message = _("""You are going to duplicate the whole content, not just to create a new version!
The new duplicated content will get a new internal number, but will lose previous versions.
""")
    message_format = 'text'


#
# Custom columns mixins
#

class WfModifiedContentColumnMixin(object):
    """Shared content modified column mixin"""

    def renderCell(self, item):
        value = self.getValue(item)
        content = get_parent(item, IWfSharedContent)
        if content is not None:
            if (IWorkflowState(content).version_id > 1) and \
                    (IZopeDublinCore(item).modified > IZopeDublinCore(content).created):
                translate = self.request.localizer.translate
                value += '<i class="fa fa-fw fa-circle txt-color-orange pull-right hint" title="{0}" ' \
                         'data-ams-hint-gravity="e"></i>'.format(translate(_("Created or modified in this version")))
        return value
