#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
from onf_website.profile.interfaces import IAdminProfile
from onf_website.workflow import WAITING_STATES

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from hypatia.interfaces import ICatalog
from onf_website.shared.common.interfaces import ISharedTool, IWfSharedContent
from onf_website.shared.common.interfaces.zmi import ISharedToolDashboardTable
from onf_website.zmi.interfaces import IDashboardMenu
from pyams_form.interfaces.form import IWidgetForm
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces import ISecurityManager
from pyams_sequence.interfaces import ISequentialIdInfo, ISequentialIdTarget, ISequentialIntIds
from pyams_skin.interfaces import IInnerPage, IPageHeader, IContentTitle
from pyams_skin.interfaces.container import ITableElementName
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowManagedContent, IWorkflowState, IWorkflow, IWorkflowVersions
from pyams_zmi.interfaces.menu import IPropertiesMenu, IContentManagementMenu, ISiteManagementMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IValues, IColumn
from zope.dublincore.interfaces import IZopeDublinCore

# import packages
from hypatia.catalog import CatalogQuery
from hypatia.query import Eq, Any
from pyams_catalog.query import CatalogResultSet
from pyams_form.form import AJAXEditForm
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, I18nColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config, ContextRequestViewAdapter
from pyams_utils.date import format_datetime, SH_DATETIME_FORMAT
from pyams_utils.list import unique
from pyams_utils.registry import query_utility, get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.view import AdminView
from pyramid.view import view_config
from z3c.form import field
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from onf_website import _


@adapter_config(context=(ISharedTool, IPyAMSLayer, Interface), provides=IContentTitle)
class SharedToolTitleAdapter(ContextRequestViewAdapter):
    """Shared tool title adapter"""

    @property
    def title(self):
        return II18n(self.context).query_attribute('title', request=self.request)


#
# Shared tool properties
#

@viewlet_config(name='properties.menu', context=ISharedTool, layer=IAdminLayer,
                manager=ISiteManagementMenu, permission='system.view', weight=1)
@viewletmanager_config(name='properties.menu', layer=IAdminLayer, provides=IPropertiesMenu)
@implementer(IPropertiesMenu)
class SharedToolPropertiesMenu(MenuItem):
    """Shared tool properties menu"""

    label = _("Properties")
    icon_class = 'fa-edit'
    url = '#properties.html'


@pagelet_config(name='properties.html', context=ISharedTool, layer=IPyAMSLayer, permission='system.view')
@implementer(IWidgetForm, IInnerPage)
class SharedToolPropertiesEditForm(AdminEditForm):
    """Shared tool properties edit form"""

    legend = _("Shared tool properties")

    fields = field.Fields(ISharedTool).omit('__parent__', '__name__')
    ajax_handler = 'properties.json'
    edit_permission = 'onf.ManageTool'


@view_config(name='properties.json', context=ISharedTool, request_type=IPyAMSLayer,
             permission='onf.ManageTool', renderer='json', xhr=True)
class SharedToolPropertiesAJAXEditForm(AJAXEditForm, SharedToolPropertiesEditForm):
    """Shared tool properties edit form, JSON renderer"""


@adapter_config(context=(ISharedTool, IAdminLayer, SharedToolPropertiesEditForm), provides=IPageHeader)
class SharedToolPropertiesHeaderAdapter(DefaultPageHeaderAdapter):
    """Shared tool properties header adapter"""

    back_url = '/admin.html#properties.html'
    back_target = None

    icon_class = 'fa fa-fw fa-edit'


#
# Shared tool control panel
#

@viewlet_config(name='dashboard.menu', context=ISharedTool, layer=IAdminLayer,
                manager=IContentManagementMenu, permission='system.view', weight=1)
@viewletmanager_config(name='dashboard.menu', layer=IAdminLayer, provides=IDashboardMenu)
@implementer(IDashboardMenu)
class SharedToolDashboardMenu(MenuItem):
    """Shared tool dashboard menu"""

    label = _("Dashboard")
    icon_class = 'fa-line-chart'
    url = '#dashboard.html'


@pagelet_config(name='dashboard.html', context=ISharedTool, layer=IPyAMSLayer, permission='system.view')
@template_config(template='templates/dashboard.pt', layer=IAdminLayer)
@implementer(IInnerPage)
class SharedToolDashboardView(AdminView):
    """Shared tool dashboard view"""

    title = _("Contents dashboard")

    def __init__(self, context, request):
        super(SharedToolDashboardView, self).__init__(context, request)
        self.waiting_table = SharedToolDashboardWaitingTable(self.context, self.request)
        self.modified_table = SharedToolDashboardModifiedTable(self.context, self.request)

    def update(self):
        super(SharedToolDashboardView, self).update()
        self.waiting_table.update()
        self.modified_table.update()


@implementer(ISharedToolDashboardTable)
class SharedToolDashboardWaitingTable(BaseTable):
    """Shared tool dashboard waiting table"""

    title = _("Contents waiting for action")
    hide_toolbar = True
    sortOn = None

    @property
    def data_attributes(self):
        attributes = super(SharedToolDashboardWaitingTable, self).data_attributes
        attributes['table'] = {'data-ams-datatable-sorting': '[]',
                               'data-ams-datatable-display-length':
                                   IAdminProfile(self.request.principal).table_page_length}
        return attributes


@adapter_config(context=(ISharedTool, IPyAMSLayer, SharedToolDashboardWaitingTable), provides=IValues)
class SharedToolDashboardWaitingValues(ContextRequestViewAdapter):
    """Shared tool dashboard waiting values adapter"""

    @property
    def values(self):
        catalog = query_utility(ICatalog)
        params = Eq(catalog['content_type'], self.context.shared_content_type) & \
                 Any(catalog['workflow_state'], WAITING_STATES)
        return unique(map(lambda x: sorted(IWorkflowVersions(x).get_versions(IWorkflowState(x).state),
                                           key=lambda y: IZopeDublinCore(y).modified, reverse=True)[0],
                          CatalogResultSet(CatalogQuery(catalog).query(params,
                                                                       sort_index='modified_date'))))


@implementer(ISharedToolDashboardTable)
class SharedToolDashboardModifiedTable(BaseTable):
    """Shared tool dashboard modified table"""

    title = _("Last modified contents (limited to 10)")
    hide_toolbar = True
    sortOn = None

    @property
    def data_attributes(self):
        attributes = super(SharedToolDashboardModifiedTable, self).data_attributes
        attributes['table'] = {'data-ams-datatable-sorting': '[]',
                               'data-ams-datatable-pagination': 'false',
                               'data-ams-datatable-display-length': 10}
        return attributes


@adapter_config(context=(ISharedTool, IPyAMSLayer, SharedToolDashboardModifiedTable), provides=IValues)
class SharedToolDashboardValues(ContextRequestViewAdapter):
    """Shared tool dashboard modified values adapter"""

    @property
    def values(self):
        catalog = query_utility(ICatalog)
        params = Eq(catalog['content_type'], self.context.shared_content_type)
        return unique(map(lambda x: IWorkflowVersions(x).get_last_versions()[0],
                          CatalogResultSet(CatalogQuery(catalog).query(params,
                                                                       limit=10,
                                                                       sort_index='modified_date',
                                                                       reverse=True))))


@adapter_config(context=(IWfSharedContent, IPyAMSLayer, ISharedToolDashboardTable), provides=ITableElementName)
class SharedToolDashboardContentNameAdapter(ContextRequestViewAdapter):
    """Shared tool dashboard content name adapter"""

    @property
    def name(self):
        return II18n(self.context).query_attribute('short_name', request=self.request)


@adapter_config(name='sequence', context=(Interface, IPyAMSLayer, ISharedToolDashboardTable), provides=IColumn)
class SharedToolDashboardSequenceColumn(I18nColumn, GetAttrColumn):
    """Shared tool dashboard sequence ID column"""

    _header = _("Unique ID")
    weight = 14

    def getValue(self, obj):
        target = get_parent(obj, ISequentialIdTarget)
        sequence = get_utility(ISequentialIntIds, name=target.sequence_name)
        return sequence.get_short_oid(ISequentialIdInfo(obj).oid, target.sequence_prefix)


@adapter_config(name='version', context=(Interface, IPyAMSLayer, ISharedToolDashboardTable), provides=IColumn)
class SharedToolDashboardVersionColumn(I18nColumn, GetAttrColumn):
    """Shared tool dashboard version column"""

    _header = _("Version")
    weight = 15

    def getValue(self, obj):
        return str(IWorkflowState(obj).version_id)


@adapter_config(name='status', context=(Interface, IPyAMSLayer, ISharedToolDashboardTable), provides=IColumn)
class SharedToolDashboardStatusColumn(I18nColumn, GetAttrColumn):
    """Shared tool dashboard status column"""

    _header = _("Status")
    weight = 20

    def getValue(self, obj):
        content = get_parent(obj, IWorkflowManagedContent)
        workflow = query_utility(IWorkflow, name=content.workflow_name)
        state = IWorkflowState(obj)
        return self.request.localizer.translate(workflow.get_state_label(state.state))


@adapter_config(name='owner', context=(Interface, IPyAMSLayer, ISharedToolDashboardTable), provides=IColumn)
class SharedToolDashboardOwnerColumn(I18nColumn, GetAttrColumn):
    """Shared tool dashboard owner column"""

    _header = _("Owner")
    weight = 30

    def getValue(self, obj):
        manager = query_utility(ISecurityManager)
        return manager.get_principal(obj.owner).title


@adapter_config(name='modified', context=(Interface, IPyAMSLayer, ISharedToolDashboardTable), provides=IColumn)
class SharedToolDashboardModifiedColumn(I18nColumn, GetAttrColumn):
    """Shared tool dashboard modified column"""

    _header = _("Last modification")
    weight = 40

    def getValue(self, obj):
        return format_datetime(tztime(IZopeDublinCore(obj).modified), SH_DATETIME_FORMAT, request=self.request)


@adapter_config(context=(ISharedTool, IPyAMSLayer, SharedToolDashboardView), provides=IPageHeader)
class SharedToolDashboardHeaderAdapter(DefaultPageHeaderAdapter):
    """Shared tool properties header adapter"""

    back_url = '/admin.html#dashboard.html'
    back_target = None

    icon_class = 'fa fa-fw fa-line-chart'
