#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from onf_website.shared.common.interfaces import IWfSharedContent, ISharedTool, ISharedContent
from pyams_form.interfaces.form import IInnerTabForm
from pyams_skin.layer import IPyAMSLayer
from pyams_workflow.interfaces import IWorkflowInfo, IWorkflowTransitionInfo, IWorkflowPublicationInfo, \
    IWorkflowCommentInfo, IWorkflowVersions, IWorkflowState, IWorkflowManagedContent, IWorkflow

# import packages
from onf_website.shared.common.zmi.summary import SharedContentSummaryForm
from onf_website.workflow import DRAFT, DELETED
from pyams_form.form import AJAXAddForm, InnerDisplayForm
from pyams_form.schema import CloseButton
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_workflow.zmi.transition import WorkflowContentTransitionForm
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import Interface

from onf_website import _


#
# Workflow summary view
#

@adapter_config(name='workflow',
                context=(IWfSharedContent, IPyAMSLayer, SharedContentSummaryForm),
                provides=IInnerTabForm)
class WfSharedContentWorkflowSummary(InnerDisplayForm):
    """Shared content workflow summary"""

    weight = 10
    tab_label = _("Workflow")
    css_class = 'form-tight'

    fields = field.Fields(IWorkflowState).omit('history') + \
             field.Fields(IWorkflowPublicationInfo)

    def updateWidgets(self, prefix=None):
        super(WfSharedContentWorkflowSummary, self).updateWidgets(prefix)
        state = IWorkflowState(self.context)
        content = get_parent(self.context, IWorkflowManagedContent)
        workflow = get_utility(IWorkflow, name=content.workflow_name)
        self.widgets['state'].value = self.request.localizer.translate(workflow.get_state_label(state.state))
        self.widgets['state_date'].value = format_datetime(tztime(state.state_date))
        info = IWorkflowPublicationInfo(self.context)
        if info.publication_date:
            self.widgets['publication_date'].value = format_datetime(tztime(info.publication_date))
        if info.first_publication_date:
            self.widgets['first_publication_date'].value = format_datetime(tztime(info.first_publication_date))
        if info.publication_effective_date:
            self.widgets['publication_effective_date'].value = format_datetime(tztime(info.publication_effective_date))
        if info.publication_expiration_date:
            self.widgets['publication_expiration_date'].value = format_datetime((tztime(info.publication_expiration_date)))


#
# Request publication form
#

class IPublicationRequestButtons(Interface):
    """Shared content publication request buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Request publication"))


@pagelet_config(name='wf-propose.html', context=IWfSharedContent, layer=IPyAMSLayer, permission='onf.ManageContent')
class PublicationRequestForm(WorkflowContentTransitionForm):
    """Shared content publication request form"""

    fields = field.Fields(IWorkflowTransitionInfo) + \
             field.Fields(IWorkflowPublicationInfo).select('publication_effective_date',
                                                           'publication_expiration_date') + \
             field.Fields(IWorkflowCommentInfo)
    buttons = button.Buttons(IPublicationRequestButtons)
    ajax_handler = 'wf-propose.json'

    def createAndAdd(self, data):
        pub_info = IWorkflowPublicationInfo(self.context)
        pub_info.publication_effective_date = data.get('publication_effective_date')
        pub_info.publication_expiration_date = data.get('publication_expiration_date')
        return super(PublicationRequestForm, self).createAndAdd(data)


@view_config(name='wf-propose.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class PublicationRequestAJAXForm(AJAXAddForm, PublicationRequestForm):
    """Shared content publication request form, JSON renderer"""


#
# Cancel publication request form
#

class IPublicationRequestCancelButtons(Interface):
    """Shared content publication request cancel buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Cancel publication request"))


@pagelet_config(name='wf-cancel-propose.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.ManageContent')
class PublicationRequestCancelForm(WorkflowContentTransitionForm):
    """Shared content publication request cancel form"""

    buttons = button.Buttons(IPublicationRequestCancelButtons)
    ajax_handler = 'wf-cancel-propose.json'


@view_config(name='wf-cancel-propose.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class PublicationRequestCancelAJAXForm(AJAXAddForm, PublicationRequestCancelForm):
    """Shared content publication request cancel form, JSON renderer"""


#
# Refuse publication form
#

class IPublicationRequestRefuseButtons(Interface):
    """Shared content publication request refuse buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Refuse publication request"))


@pagelet_config(name='wf-refuse.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.PublishContent')
class PublicationRequestRefuseForm(WorkflowContentTransitionForm):
    """Shared content publication request refuse form"""

    buttons = button.Buttons(IPublicationRequestRefuseButtons)
    ajax_handler = 'wf-refuse.json'


@view_config(name='wf-refuse.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.PublishContent', renderer='json', xhr=True)
class PublicationRequestRefuseAJAXForm(AJAXAddForm, PublicationRequestRefuseForm):
    """Shared content publication request refuse form, JSON renderer"""


#
# Publish form
#

class IPublicationButtons(Interface):
    """Shared content publication buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Publish"))


@pagelet_config(name='wf-publish.html', context=IWfSharedContent, layer=IPyAMSLayer, permission='onf.PublishContent')
class PublishForm(WorkflowContentTransitionForm):
    """Shared content publication form"""

    fields = field.Fields(IWorkflowTransitionInfo) + \
             field.Fields(IWorkflowPublicationInfo).select('publication_effective_date',
                                                           'publication_expiration_date') + \
             field.Fields(IWorkflowCommentInfo)
    buttons = button.Buttons(IPublicationButtons)
    ajax_handler = 'wf-publish.json'

    def updateWidgets(self, prefix=None):
        super(PublishForm, self).updateWidgets(prefix)
        pub_info = IWorkflowPublicationInfo(self.context)
        if ('publication_effective_date' in self.widgets) and pub_info.publication_effective_date:
            self.widgets['publication_effective_date'].value = pub_info.publication_effective_date.strftime('%d/%m/%y %H:%M')
        if ('publication_expiration_date' in self.widgets) and pub_info.publication_expiration_date:
            self.widgets['publication_expiration_date'].value = pub_info.publication_expiration_date.strftime('%d/%m/%y %H:%M')

    def createAndAdd(self, data):
        pub_info = IWorkflowPublicationInfo(self.context)
        pub_info.publication_effective_date = data.get('publication_effective_date')
        pub_info.publication_expiration_date = data.get('publication_expiration_date')
        return super(PublishForm, self).createAndAdd(data)


@view_config(name='wf-publish.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.PublishContent', renderer='json', xhr=True)
class PublicationAJAXForm(AJAXAddForm, PublishForm):
    """Shared content publication form, JSON renderer"""


#
# Publication retire request form
#

class IPublicationRetireRequestButtons(Interface):
    """Shared content publication retire request buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Request retire"))


@pagelet_config(name='wf-retiring.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.ManageContent')
class PublicationRetireRequestForm(WorkflowContentTransitionForm):
    """Shared content publication request refuse form"""

    buttons = button.Buttons(IPublicationRetireRequestButtons)
    ajax_handler = 'wf-retiring.json'


@view_config(name='wf-retiring.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class PublicationRetireRequestAJAXForm(AJAXAddForm, PublicationRetireRequestForm):
    """Shared content publication retire request form, JSON renderer"""


#
# Publication retire cancel form
#

class IPublicationRetireCancelButtons(Interface):
    """Shared content publication retire request cancel buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Cancel retire request"))


@pagelet_config(name='wf-cancel-retiring.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.ManageContent')
class PublicationRetireCancelForm(WorkflowContentTransitionForm):
    """Shared content publication retire request cancel form"""

    buttons = button.Buttons(IPublicationRetireCancelButtons)
    ajax_handler = 'wf-cancel-retiring.json'


@view_config(name='wf-cancel-retiring.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class PublicationRetireCancelAJAXForm(AJAXAddForm, PublicationRetireCancelForm):
    """Shared content publication retire request cancel form, JSON renderer"""


#
# Publication retire form
#

class IPublicationRetireButtons(Interface):
    """Shared content publication retire buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Retire"))


@pagelet_config(name='wf-retire.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.PublishContent')
class PublicationRetireForm(WorkflowContentTransitionForm):
    """Shared content publication retire form"""

    buttons = button.Buttons(IPublicationRetireButtons)
    ajax_handler = 'wf-retire.json'


@view_config(name='wf-retire.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.PublishContent', renderer='json', xhr=True)
class PublicationRetireAJAXForm(AJAXAddForm, PublicationRetireForm):
    """Shared content publication retire form, JSON renderer"""


#
# Publication archive request form
#

class IPublicationArchiveRequestButtons(Interface):
    """Shared content publication archive request buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Request archive"))


@pagelet_config(name='wf-archiving.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.ManageContent')
class PublicationArchiveRequestForm(WorkflowContentTransitionForm):
    """Shared content publication request archive form"""

    buttons = button.Buttons(IPublicationArchiveRequestButtons)
    ajax_handler = 'wf-archiving.json'


@view_config(name='wf-archiving.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class PublicationArchiveRequestAJAXForm(AJAXAddForm, PublicationArchiveRequestForm):
    """Shared content publication archive request form, JSON renderer"""


#
# Publication archive cancel form
#

class IPublicationArchiveCancelButtons(Interface):
    """Shared content publication archive request cancel buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Cancel archive request"))


@pagelet_config(name='wf-cancel-archiving.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.ManageContent')
class PublicationArchiveCancelForm(WorkflowContentTransitionForm):
    """Shared content publication archive request cancel form"""

    buttons = button.Buttons(IPublicationArchiveCancelButtons)
    ajax_handler = 'wf-cancel-archiving.json'


@view_config(name='wf-cancel-archiving.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class PublicationArchiveCancelAJAXForm(AJAXAddForm, PublicationArchiveCancelForm):
    """Shared content publication archive request cancel form, JSON renderer"""


#
# Publication archive form
#

class IPublicationArchiveButtons(Interface):
    """Shared content publication archive buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Archive"))


@pagelet_config(name='wf-archive.html', context=IWfSharedContent, layer=IPyAMSLayer,
                permission='onf.PublishContent')
class PublicationArchiveForm(WorkflowContentTransitionForm):
    """Shared content publication archive form"""

    buttons = button.Buttons(IPublicationArchiveButtons)
    ajax_handler = 'wf-archive.json'


@view_config(name='wf-archive.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.PublishContent', renderer='json', xhr=True)
class PublicationArchiveAJAXForm(AJAXAddForm, PublicationArchiveForm):
    """Shared content publication archive form, JSON renderer"""


#
# Clone form
#

class ISharedContentCloneButtons(Interface):
    """Shared content clone buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Create new version"))


@pagelet_config(name='wf-clone.html', context=IWfSharedContent, layer=IPyAMSLayer, permission='onf.CreateContent')
class SharedContentCloneForm(WorkflowContentTransitionForm):
    """Shared content clone form"""

    buttons = button.Buttons(ISharedContentCloneButtons)
    ajax_handler = 'wf-clone.json'

    def createAndAdd(self, data):
        info = IWorkflowInfo(self.context)
        return info.fire_transition_toward(DRAFT, comment=data.get('comment'))


@view_config(name='wf-clone.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.CreateContent', renderer='json', xhr=True)
class SharedContentCloneAJAXForm(AJAXAddForm, SharedContentCloneForm):
    """Shared content clone form, JSON rendener"""

    def get_ajax_output(self, changes):
        return {'status': 'redirect',
                'location': absolute_url(changes, self.request, 'admin.html#properties.html')}


#
# Delete form
#

class ISharedContentDeleteButtons(Interface):
    """Shared content delete form buttons"""

    close = CloseButton(name='close', title=_("Close"))
    action = button.Button(name='action', title=_("Delete version"))


@pagelet_config(name='wf-delete.html', context=IWfSharedContent, layer=IPyAMSLayer, permission='onf.ManageContent')
class SharedContentDeleteForm(WorkflowContentTransitionForm):
    """Shared content delete form"""

    buttons = button.Buttons(ISharedContentDeleteButtons)
    ajax_handler = 'wf-delete.json'

    def createAndAdd(self, data):
        state = IWorkflowState(self.context)
        if state.version_id == 1:  # remove the first and only version => remove all
            content = get_parent(self.context, ISharedContent)
            self.__target = get_parent(content, ISharedTool)
            del content.__parent__[content.__name__]
        else:
            self.__target = get_parent(self.context, ISharedTool)
            IWorkflowVersions(self.context).remove_version(state.version_id, state=DELETED, comment=data.get('comment'))


@view_config(name='wf-delete.json', context=IWfSharedContent, request_type=IPyAMSLayer,
             permission='onf.ManageContent', renderer='json', xhr=True)
class SharedContentDeleteAJAXForm(AJAXAddForm, SharedContentDeleteForm):
    """Shared content delete form, JSON rendener"""

    def get_ajax_output(self, changes):
        return {'status': 'redirect',
                'location': absolute_url(self._SharedContentDeleteForm__target, self.request, 'admin.html')}
